from django.db.models import Model as djangoModel
from django.core.exceptions import ObjectDoesNotExist


class FieldError(ValueError):
    pass


class ForeignKeyFieldError(FieldError):

    def __init__(self, msg, model, value):
        self.model = model
        self.value = value
        self.msg = msg
        super(ForeignKeyFieldError, self).__init__(self.msg)


class Field(object):
    position = 0

    def __init__(self, **kwargs):
        if 'row_num' in kwargs:
            self.position = kwargs.pop('row_num')
        else:
            self.position = Field.position
            Field.position += 1
        if 'match' in kwargs:
            self.match = kwargs.pop('match')
        if 'transform' in kwargs:
            self.transform = kwargs.pop('transform')
        if 'validator' in kwargs:
            self.validator = kwargs.pop('validator')
        if 'multiple' in kwargs:
            self.has_multiple = kwargs.pop('multiple')
        if 'prepare' in kwargs:
            self.prepare = kwargs.pop('prepare')
        if 'keys' in kwargs and isinstance(self, ComposedKeyField):
            self.keys = kwargs.pop('keys')
        if len(kwargs) > 0:
            raise ValueError("Arguments %s unexpected" % kwargs.keys())

    def get_prep_value(self, value):
        try:
            if hasattr(self, "prepare"):
                value = self.prepare(value)
            value = self.to_python(value)
            if hasattr(self, "transform"):
                value = self.transform(value)
            if hasattr(self, "validator"):
                validator = self.validator()
                if not validator.validate(value):
                    raise FieldError(validator.__class__.validation_message)
            return value
        except FieldError, e:
            raise e
        except ValueError:
            raise ValueError("Value \'%s\' in columns %d does not match the expected type %s" %
                              (value, self.position + 1, self.__class__.field_name))


class IntegerField(Field):

    field_name = "Integer"

    def to_python(self, value):
        return int(value)


class BooleanField(Field):

    field_name = "Boolean"

    def default_is_true_method(self, value):
        return value.lower() == "true"

    def __init__(self,  *args, **kwargs):
        if 'is_true' in 'kwargs':
            self.is_true_method = kwargs.pop('is_true')
        else:
            self.is_true_method = self.default_is_true_method
        super(BooleanField, self).__init__( *args, **kwargs)
        
    def to_python(self, value):
        return self.is_true_method(value)


class CharField(Field):

    field_name = "String"

    def to_python(self, value):
        return value


class FloatField(Field):
    field_name = "A float number"

    def to_python(self, value):
        return float(value)


class IgnoredField(Field):
    field_name = "Ignore the value"

    def to_python(self, value):
        return None


class ForeignKey(Field):
    field_name = "not defined"

    def __init__(self, *args, **kwargs):
        self.pk = kwargs.pop('pk', 'pk')
        if len(args) < 1:
            raise ValueError("You should provide a Model as the first argument.")
        self.model = args[0]
        try:
            if not issubclass(self.model, djangoModel):
                raise TypeError("The first argument should be a django model class.")
        except TypeError, e:
            raise TypeError("The first argument should be a django model class.")
        super(ForeignKey, self).__init__(**kwargs)

    def to_python(self, value):
        try:
            return self.model.objects.get(**{self.pk: value})
        except ObjectDoesNotExist, e:
            raise ForeignKeyFieldError("No match found for %s" % self.model.__name__, self.model.__name__, value)
        except MultipleObjectsReturned, e:
            raise ForeignKeyFieldError("Multiple match found for %s" % self.model.__name__, self.model.__name__, value)

class ComposedKeyField(ForeignKey):

    def to_python(self, value):
        try:
            return self.model.objects.get(**value)
        except ObjectDoesNotExist, e:
            raise ForeignKeyFieldError("No match found for %s" % self.model.__name__, self.model.__name__, value)



from django.test import TestCase
from fields import *
from model import CsvModel, CsvDbModel, ImproperlyConfigured, CsvException, CsvDataException, TabularLayout, SkipRow
from myTestModel.models import *


class TestCsvModel(CsvModel):
    nom = CharField()
    age = IntegerField()
    taille = FloatField()

    class Meta:
        delimiter = ";"

    test_data = ["Roger", "10", "1.8"]


class TestCsvError(CsvModel):
    nom = CharField()
    age = IntegerField()
    taille = FloatField()

    class Meta:
        delimiter = ";"

    test_data = ["10", "Roger", "1.8"]


class TestCsvSingleData(CsvModel):
    single = CharField()


class TestCsvNoDelimiter(CsvModel):
    nom = CharField()
    age = IntegerField()

    test_data = ["10", "Roger"]


class TestCsvNoField(CsvModel):
    test_data = ["tata", "yoyo"]


class TestCsvMultipleLine(TestCsvModel):
    nom = CharField()
    age = IntegerField()
    taille = FloatField()

    test_data = ["Roger;10;1.8", "Janette;12;1.7"]


class TestCsvDBModel(TestCsvModel):
    class Meta:
        delimiter = ";"
        dbModel = MyModel


class TestCsvDBUnmatchingModel(CsvModel):
    name = CharField(match='nom')
    age = IntegerField()
    taille = FloatField()

    class Meta:
        delimiter = ";"
        dbModel = MyModel



class TestCsvExitOnFailure(TestCsvModel):
    class Meta:
        silent_failure = True
        delimiter = ";"
        dbModel = MyModel

    test_data = ["Roger;Error;1.8", "Janette;12;1.7"]
    
class TestCsvDBOnlyModel(CsvDbModel):
    class Meta:
        dbModel = MyModel
        delimiter = ";"
#        exclude = ['id']
        
    test_data = ["Janette;12;1.7","Roger;18;1.8"]

        
class TestCsvDbForeignInvalid(CsvDbModel):
    foreign = ForeignKey(MyModel)
    
    class Meta:
        dbModel = MyModelWithForeign


class TestCsvImporter(TestCase):
    def test_has_delimiter(self):
        self.assertTrue(TestCsvModel.has_class_delimiter())
        self.assertFalse(TestCsvNoDelimiter.has_class_delimiter())

    def test_is_db_model(self):
        self.assertFalse(TestCsvModel.is_db_model())
        self.assertTrue(TestCsvDBModel.is_db_model())

    def test_basic(self):
        test = TestCsvModel(data=TestCsvModel.test_data)
        self.assertEquals(test.nom, 'Roger')
        self.assertEquals(test.age, 10)
        self.assertEquals(test.taille, 1.8)

    def test_value_error(self):
        self.assertRaises(ValueError, TestCsvError, data=TestCsvError.test_data)

    def test_single(self):
        test = TestCsvSingleData(data=["RogerAgain"])
        self.assertEquals(test.single, "RogerAgain")

    def test_no_delimiter(self):
        self.assertRaises(ImproperlyConfigured, TestCsvNoDelimiter, data=TestCsvError.test_data)

    def test_no_field(self):
        self.assertRaises(ImproperlyConfigured, TestCsvNoField, data=TestCsvNoField.test_data)

    def test_multiple_lines(self):
        test = TestCsvMultipleLine.import_data(data=TestCsvMultipleLine.test_data)
        line1 = test[0]
        self.assertEquals(line1.nom, 'Roger')
        self.assertEquals(line1.age, 10)
        self.assertEquals(line1.taille, 1.8)

        line2 = test[1]
        self.assertEquals(line2.nom, 'Janette')
        self.assertEquals(line2.age, 12)
        self.assertEquals(line2.taille, 1.7)

        index = 0
        for line in test:
            self.assertEquals(line, test[index])
            index += 1

    def test_real_file(self):
        file = open("test/csv1.csv")
        test = TestCsvModel.import_from_file(file)
        line1 = test[0]
        self.assertEquals(line1.nom, 'Roger')
        self.assertEquals(line1.age, 10)
        self.assertEquals(line1.taille, 1.8)

    def test_db_model(self):
        test = TestCsvDBModel.import_from_filename("test/csv2.csv")
        self.assertEquals(MyModel.objects.all().count(), 2)

    def test_db_unmatching_model(self):
        test = TestCsvDBUnmatchingModel.import_from_filename("test/csv1.csv")
        self.assertEquals(MyModel.objects.all().count(), 1)

    def test_field_unexpected_attributes(self):
        def create_unexpected_model():
            #class TestCsvUnexpectedAttributes(CsvModel):
            #    name = CharField(unexpected=True)
            return type('TestCsvDBUnmatchingModel', CsvModel, {'name': CharField(unexpected=True)})

        self.assertRaises(ValueError, create_unexpected_model)

    def test_exit_on_failure(self):
        test = TestCsvExitOnFailure.import_data(TestCsvExitOnFailure.test_data)
        self.assertEquals(MyModel.objects.all().count(), 1)

    def test_with_header(self):
        class TestCsvWithHeader(TestCsvModel):
            class Meta:
                delimiter = ";"
                has_header = True
                dbModel = MyModel

            test_data = ["Name;Age;length", "Roger;10;1.8", "Janette;12;1.7"]

        
        
        test = TestCsvWithHeader.import_data(TestCsvWithHeader.test_data)
        self.assertEquals(MyModel.objects.all().count(), 2)
        
    def test_direct_to_db(self):
        test = TestCsvDBOnlyModel.import_data(TestCsvDBOnlyModel.test_data)
        self.assertEquals(MyModel.objects.all().count(), 2)
        myModel = MyModel.objects.all()[0]
        self.assertEquals(myModel.nom, "Janette")
        
        self.assertEquals(myModel.age, 12)
        self.assertEquals(myModel.taille, 1.7)
        
    def test_foreign_key_model(self):
        class TestCsvDbForeign(CsvModel):
            foreign = ForeignKey(MyModel)
    
            class Meta:
                dbModel = MyModelWithForeign
                delimiter = ","
        
        my_model = MyModel.objects.create(nom="Gigi", age=10, taille= 1.2)
        test = TestCsvDbForeign.import_data(["%d" % my_model.id])
        self.assertEquals(MyModelWithForeign.objects.all().count(), 1)
        
        
    def test_with_invalid_foreign(self):
        self.assertRaises(ImproperlyConfigured,TestCsvDbForeignInvalid,[1])
        
    def test_transform_foreign(self):
        class TestCsvDbForeignFollow(CsvModel):
            foreign_csv = ForeignKey(MyModelWithForeign,transform= lambda x:x.foreign,match="foreign")
    
            class Meta:
                dbModel = OtherForeign
                delimiter = ","
        
        my_model = MyModel.objects.create(nom="Gigi", age=10, taille= 1.2)
        foreign = MyModelWithForeign.objects.create(foreign=my_model)
        test = TestCsvDbForeignFollow.import_data(["%d" % foreign.id] )
        self.assertEquals(my_model, test[0].foreign_csv)
        
    def test_import_from_file_sniffer(self):
        
        class TestCsvWithHeader(TestCsvModel):
            class Meta:
                dbModel = MyModel

        test = TestCsvWithHeader.import_from_filename("test/csv3.csv")
        self.assertEquals(MyModel.objects.all().count(), 23)
        
    def test_error_message_foreign(self):
        class TestCsvDbForeign(CsvModel):
            foreign = ForeignKey(MyModel)
    
            class Meta:
                dbModel = MyModelWithForeign
                delimiter = ","
                
        test_data_template = "%d,10,1"
        my_model = MyModel.objects.create(nom="Gigi", age=10, taille= 1.2)
        data = [test_data_template % my_model.id,test_data_template % (my_model.id +999) ]
        try:
            test = TestCsvDbForeign.import_data(data)
        except CsvException , e:
            self.assertEquals(e.message, u'Line 2: No match found for MyModel')
        else:
            self.assertTrue(False,"No valueError raised")
            
    def test_error_message_too_many_field(self):
        try:
            test = TestCsvModel.import_data(['1,error,12'])
        except CsvException , e:
            self.assertEquals(e.message, u'Line 1: Number of fields invalid')
        else:
            self.assertTrue(False,"No valueError raised")
            
    def test_error_message_integer_field(self):
        try:
            test = TestCsvModel.import_data(['1;error;12'])
        except CsvException , e:
            self.assertEquals(e.message, 
                               u"Line 1: Value 'error' in columns 2 does not match the expected type Integer" )
        else:
            self.assertTrue(False,"No valueError raised")
            
            
    def test_validator(self):
        
        class CsvValidator(CsvModel):
            
            class Meta:
                delimiter = ";"
                
            class Validate:
                validation_message = "Your value should be 10"
                def validate(self,value):
                    return value == 10
                
            age = IntegerField(validator=Validate)
            
        self.assertRaises(CsvDataException, CsvValidator.import_data,['11'])
        try:
            CsvValidator.import_data(['10'])
        except CsvDataException,e :
            self.assertTrue(False,"No exception should be raised")
        self.assertTrue(True)
        
        
        
    def test_multiple_fields(self):
        
        class CsvMultiple(CsvModel):
            
            nom = CharField()
            note = IntegerField(multiple = True)
            
            class Meta:
                delimiter = ";"
                dbModel = MultipleModel
                
        test_data = ["josette;18;12;8"]
            
        test = CsvMultiple.import_data(test_data)
        self.assertEquals(MultipleModel.objects.count(),3)
        
    
    def test_tabular_layout(self):
        
        class CsvTabular(CsvModel):
            
            nom = CharField()
            age = IntegerField()
            taille = FloatField()
            
            class Meta:
                delimiter = ";"
                layout = TabularLayout
                dbModel = MyModel
                
        test_data = [";8;12;18","Janette;1.2;1.4;1.6","popeye;0.8;1.0;1.3"]
        test = CsvTabular.import_data(test_data)
        self.assertEquals(MyModel.objects.all().count(),6)
        
    def test_prepare(self):
        
        def upper(name):
            return name.upper()
        
        class CsvPrepare(CsvModel):
            
            nom = CharField(prepare=upper)
            age = IntegerField()
            taille = FloatField()
            
            class Meta:
                delimiter = ";"
                dbModel = MyModel
        
        test_data = ["Janette;12;1.7","Roger;18;1.8"]
        test = CsvPrepare.import_data(test_data)
        self.assertEquals(test[0].nom, "JANETTE")
        self.assertEquals(test[1].nom, "ROGER")
        
    def test_ignore_field(self):
        
        class IgnoreCsv(CsvModel):
            nom = CharField()
            ignored = IgnoredField()
            age = IntegerField()
            taille = FloatField()
            
             
            class Meta:
                delimiter = ";"
                dbModel = MyModel
            
        test_data = ["Janette;Dont care;12;1.7","Roger;Dont care;18;1.8"]
        test = IgnoreCsv.import_data(test_data)
        self.assertEquals(MyModel.objects.all().count(),2)
        
    def test_skip_row(self):
        
        def skip_janette(name):
            if name == "Janette":
                raise SkipRow()
            return name
        
        class SkipRowCsv(CsvModel):
            nom = CharField(prepare=skip_janette)
            age = IntegerField()
            taille = FloatField()
            
            class Meta:
                delimiter = ";"
                dbModel = MyModel
            
        test_data = ["Janette;12;1.7","Roger;18;1.8"]
        test = SkipRowCsv.import_data(test_data)
        self.assertEquals(MyModel.objects.all().count(),1)
        
    def test_multiple_key_foreign(self):
        
        class ComposedForeignKeyCsv(CsvModel):
            key_1 = IntegerField()
            key_2 = IntegerField()
            composed_key_foreign = ComposedKeyField(ComposedKeyForeign,keys=["key_1","key_2"])
            
            class Meta:
                delimiter = ";"
                dbModel = ComposedKey
                
        c0 = ComposedKeyForeign.objects.create(key_1=1, key_2=1)
        c1 = ComposedKeyForeign.objects.create(key_1=1, key_2=2)
        test_data = ["1;1","1;2"]
        test = ComposedForeignKeyCsv.import_data(test_data)
        self.assertEquals(c0, test[0].composed_key_foreign)
        self.assertEquals(c1, test[1].composed_key_foreign)

    def test_update(self):
        
        class TestUpdateCsv(CsvModel):
            nom = CharField()
            age = IntegerField()
            taille = FloatField()
            
            class Meta:
                dbModel = MyModel
                delimiter = ";"
                update = {'keys':["nom","age"]} 
                
        test_data = ["Janette;12;1.0","Janette;12;2.0"]
        self.assertEquals(MyModel.objects.count(),0)
        test = TestUpdateCsv.import_data(test_data)
        self.assertEquals(MyModel.objects.count(),1)
        self.assertEquals(MyModel.objects.all()[0].taille, 2.0)
        
    def test_update_only(self):
        
        class TestUpdateOnlyCsv(CsvModel):
            nom = CharField()
            age = IntegerField()
            taille = FloatField()
            poids = FloatField()
            
            class Meta:
                dbModel = MyModelBis
                delimiter = ";"
                update = {'keys':["nom","age"],'update':['poids']} 
                
        test_data = ["Janette;12;1.0;1.0","Janette;12;2.0;2.0"]
        self.assertEquals(MyModelBis.objects.count(),0)
        test = TestUpdateOnlyCsv.import_data(test_data)
        self.assertEquals(MyModelBis.objects.count(),1)
        self.assertEquals(MyModelBis.objects.all()[0].taille, 1.0)
        self.assertEquals(MyModelBis.objects.all()[0].poids, 2.0)


    def test_update_and_extra(self):
        
        class TestUpdateOnlyExtraCsv(CsvModel):
            nom = CharField()
            age = IntegerField()
            taille = FloatField()
            poids = FloatField()
            bool = BooleanField()
            
            class Meta:
                dbModel = MyModelTer
                delimiter = ";"
                update = {'keys': ["nom","age"], 'update': ['poids']}

                
        test_data = ["Janette;12;1.0;1.0", "Janette;12;2.0;2.0"]
        test = TestUpdateOnlyExtraCsv.import_data(test_data, extra_fields=["True"])
        self.assertEquals(MyModelTer.objects.count(), 1)
        self.assertEquals(MyModelTer.objects.all()[0].taille, 1.0)
        self.assertEquals(MyModelTer.objects.all()[0].poids, 2.0)
        self.assertTrue(MyModelTer.objects.all()[0].bool)
        
        test_data2 = ["Jojo;11;1.0;1.0"]
        test = TestUpdateOnlyExtraCsv.import_data(test_data2, extra_fields=["false"])
        self.assertEquals(test[0].bool, False)
        
        test = TestUpdateOnlyExtraCsv.import_data(test_data2, extra_fields=["True"])
        self.assertFalse(MyModelTer.objects.get(nom="Jojo").bool)    
        


class TestFields(TestCase):
    
    def test_foreign_key(self):
        self.assertRaises(ValueError,ForeignKey)
        self.assertRaises(TypeError,ForeignKey,10)
        field = ForeignKey(MyModel)
        myModel = MyModel.objects.create(nom="jojo",age="10",taille=1.5)
        self.assertEquals(field.to_python(myModel.id), myModel)
        
    def test_foreign_other_pk(self):
        field = ForeignKey(MyModel2,pk="other_pk")
        myModel2 = MyModel2.objects.create(other_pk=10)
        self.assertEquals(field.to_python(myModel2.other_pk), myModel2)
        
    def test_error_message(self):
        field = ForeignKey(MyModel2,pk="other_pk")
        myModel2 = MyModel2.objects.create(other_pk=999)
        try:
            field.to_python(666)
        except ValueError, e:
            self.assertEquals(e.message, 'No match found for MyModel2')
        else:
            self.assertTrue(False,"No exception raised")
        self.assertEquals(field.to_python(myModel2.other_pk), myModel2)
        
class TestImporter(TestCase):
    
    
    def test_extra_fields(self):
        
        class TestCsvExtraFields(CsvModel):
            nom = CharField()
            age = IntegerField()
            taille = FloatField()
            extra_value = CharField()
            
            class Meta:
                delimiter = ";"
                
            test_data = ["Janette;12;1.7","Roger;18;1.8"]
                
        test = TestCsvExtraFields.import_data(TestCsvExtraFields.test_data,extra_fields=["extra"])
        self.assertEquals(test[0].extra_value, "extra")
        self.assertEquals(test[1].extra_value, "extra")
        
    def test_positionnal_extra_value(self):
        
        class TestCsvExtraFieldsPositionnal(CsvModel):
            extra_value = CharField()
            nom = CharField()
            age = IntegerField()
            taille = FloatField()
            
            class Meta:
                delimiter = ";"
                
            test_data = ["Janette;12;1.7","Roger;18;1.8"]
                
        test = TestCsvExtraFieldsPositionnal.import_data(TestCsvExtraFieldsPositionnal.test_data,extra_fields=[{'value':"extra",'position':0}])
        self.assertEquals(test[0].extra_value, "extra")
        self.assertEquals(test[1].extra_value, "extra")
        
            

        




import csv
from django.db.models.fields import Field as DjangoField
from fields import Field, ForeignKeyFieldError, IgnoredField, ComposedKeyField
from django.core.exceptions import ValidationError

class ImproperlyConfigured(Exception):
    pass

class CsvException(Exception):
    pass
        
class CsvDataException(CsvException):
    def __init__(self, line, error=None ,field_error=None):
        self.line = line + 1
        self.error = error
        self.field_error = field_error
        if self.error:
            Exception.__init__(self,u"Line %d: %s" % (self.line ,self.error))
        elif self.field_error:
            Exception.__init__(self,u"Line %d: %s" % (self.line ,self.field_error))
            
class CsvFieldDataException(CsvDataException):
    def __init__(self, line, field_error, model, value):
        self.model = model
        self.value = value
        CsvDataException.__init__(self,line, field_error=field_error)
        
class SkipRow(Exception): pass
            
        
class CsvModel(object):

    def get_fields(self):
        all_cls_dict = {}
        all_cls_dict.update(self.cls.__dict__)
        for klass in self.cls.__bases__:
            all_cls_dict.update(klass.__dict__)
        attributes = [(attr, all_cls_dict[attr] ) for attr in all_cls_dict if isinstance(all_cls_dict[attr], Field)]
        sorted_field = sorted(attributes,key=lambda attrs: attrs[1].position)
        return sorted_field


    def get_value(self, attr_name, field, value):
        self.__dict__[attr_name] = field.get_prep_value(value)
        self.field_matching_name = field.__dict__.get("match", attr_name)
#        values[field_matching_name] = field.get_prep_value(value)
        return field.get_prep_value(value)



    def update_object(self, dict_values, object, update_dict):
        new_dict_values = {}
        if 'update' in update_dict:
            # Update the object for the value un update_dict['update']
            # only
            for field_name in update_dict['update']:
                new_dict_values[field_name] = dict_values[field_name]
        else:
            new_dict_values = dict_values
        for field_name in new_dict_values:
            attr = setattr(object, field_name, new_dict_values[field_name])
        object.save()

    def base_create_model(self, model, **dict_values):
        if self.cls.has_update_method():
            keys = None
            update_dict = self.cls.Meta.update
            try:
                keys = update_dict['keys']
            except KeyError:
                raise ImproperlyConfigured("The update dict should contains a keys value")
            filter_values = {}
            for key in keys:
                filter_values.update({key:dict_values[key]})
            object = None
            try:
                object = model.objects.get(**filter_values)
            except model.DoesNotExist:
                object = model.objects.create(**dict_values)
            except model.MultipleObjectsReturned:
                raise ImproperlyConfigured("Multiple values returned for the update key %s. Keys provide are not unique" % filter_values)
            else:
                self.update_object(dict_values, object, update_dict)
        else:
            model.objects.create(**dict_values)


    def create_model_instance(self, values):
        model = self.cls.Meta.dbModel
        if self.multiple_creation_field:
            if self.multiple_creation_field:
                multiple_values = values.pop(self.multiple_creation_field)
                for value in multiple_values:
                    dict_values = values.copy()
                    dict_values[self.multiple_creation_field] = value
                    self.base_create_model( model, **dict_values)
                    
        else:
            self.base_create_model( model, **values)

    def __init__(self,data,delimiter=None):
        self.delimiter = delimiter
        self.cls = self.__class__
        self.attrs = self.get_fields()
        self.validate()
        values = {}
        silent_failure = self.cls.silent_failure()
        load_failed = False
        self.multiple_creation_field = None
        composed_fields = []
        index_offset = 0
        for (attr_name,field),position in zip(self.attrs,range(len(self.attrs))):
            field.position = position
            if isinstance(field, ComposedKeyField):
                    composed_fields.append(field)
                    index_offset += 1
                    continue
            if self.cls.has_class_delimiter() or delimiter:
                value = data[position - index_offset]
            else:
                value = data[0]
            try:
                if isinstance(field, IgnoredField):
                    continue
                if hasattr(field,'has_multiple') and field.has_multiple:
                    remaining_data = data[position:]
                    multiple_values = []
                    for data in remaining_data:
                        multiple_values.append(self.get_value(attr_name, field, data))
                    values[self.field_matching_name] = multiple_values
                    self.multiple_creation_field = self.field_matching_name
                else:
                    values[self.field_matching_name] = self.get_value(attr_name, field, value)
            except ValueError,e :
                if silent_failure:
                    raise SkipRow()
                else:
                    raise e
        if self.cls.is_db_model() and not load_failed:
            for field in composed_fields:
                keys = {}
                for key in field.keys:
                    keys[key]=values.pop(key)
                values[self.field_matching_name] = self.get_value(attr_name, field, keys)
            self.create_model_instance(values)

    def validate(self):
        if len(self.attrs)==0:
            raise ImproperlyConfigured("No field defined. Should have at least one field in the model.")
        if not self.cls.has_class_delimiter() and not self.delimiter and len(self.attrs)>1:
            raise ImproperlyConfigured("More than a single field and no delimiter defined. You should define a delimiter.")
        

    @classmethod
    def is_db_model(cls):
        return hasattr(cls,"Meta") and hasattr(cls.Meta,"dbModel") and cls.Meta.dbModel

    @classmethod
    def has_class_delimiter(cls):
        return hasattr(cls,"Meta") and hasattr(cls.Meta,"delimiter")
    
    @classmethod
    def has_header(cls):
        return hasattr(cls,"Meta") and hasattr(cls.Meta,"has_header") and cls.Meta.has_header
    
    @classmethod
    def has_update_method(cls):
        has_update = hasattr(cls,"Meta") and hasattr(cls.Meta,"update")
        if has_update and not cls.is_db_model():
            raise ImproperlyConfigured("You should define a model when using the update option")
        return has_update
        
    @classmethod
    def silent_failure(cls):
        if not hasattr(cls,"Meta") or not hasattr(cls.Meta,"silent_failure"):
            return False
        return cls.Meta.silent_failure

    @classmethod
    def import_data(cls,data,extra_fields=[]):
        importer =  CsvImporter(csvModel=cls,extra_fields=extra_fields)
        return importer.import_data(data)
    
    @classmethod
    def import_from_filename(cls,filename,extra_fields=[]):
        importer =  CsvImporter(csvModel=cls,extra_fields=extra_fields)
        return importer.import_from_filename(filename)
    
    @classmethod
    def import_from_file(cls,file,extra_fields=[]):
        importer =  CsvImporter(csvModel=cls,extra_fields=extra_fields)
        return importer.import_from_file(file)
    


class CsvDbModel(CsvModel):
        
    def validate(self):
        if not self.cls.is_db_model():
            raise ImproperlyConfigured("dbModel attribute is missing or wrongly configured in the CsvDbModel class.")
        
    def get_fields(self):
        cls_attrs = super(CsvDbModel,self).get_fields()
        if len(cls_attrs) != 0:
            raise ImproperlyConfigured("A Db model should not have any csv field defined.")
        cls = self.__class__
        attrs = []
        if cls.is_db_model():
            model = cls.Meta.dbModel
            for field in model._meta.fields:
                attrs.append((field.name,field))
        excluded_fields = cls.get_exclusion_fields()
        attrs_filtered = [attr for attr in attrs if attr[0] not in excluded_fields]
        return attrs_filtered
    
    @classmethod
    def get_exclusion_fields(cls):
        list_exclusion = []
        if hasattr(cls,"Meta") and hasattr(cls.Meta,"exclude"):
            list_exclusion.append(*cls.Meta.exclude)
        if 'id' not in list_exclusion:
            list_exclusion.append('id')
        return list_exclusion
            
class LinearLayout(object):
    
    def process_line(self, lines, line, model,delimiter):
        lines.append(model(data=line,delimiter=delimiter))
        
class TabularLayout(object):
    
    def __init__(self):
        self.line_no = 0
        self.column_no = 1
        self.headers = None
    
    def process_line(self, lines, line, model,delimiter):
        if self.line_no == 0:
            self.headers = line
            self.line_no += 1
        else:
            for data in line[1:]:
                inline_data = [line[0],self.headers[self.column_no],data]
                lines.append(model(data=inline_data,delimiter=delimiter))
                self.column_no += 1
            self.column_no = 1
            
class CsvImporter(object):

    def __init__(self, csvModel, extra_fields=[], layout = None):
        self.csvModel = csvModel
        self.extra_fields = extra_fields
        self.dialect = None
        self.delimiter = None
        if not layout:
            if hasattr(self.csvModel, 'Meta') and hasattr(self.csvModel.Meta, 'layout'):
                self.layout = self.csvModel.Meta.layout()
            else:
                self.layout = LinearLayout()


    def process_extra_fields(self, data, line):
        data_length = len(line)
        if self.extra_fields:
            extra_field_index = 0
            for value in self.extra_fields:
                if isinstance(value, str):
                    line.append(value)
                elif isinstance(value, dict):
                    position = value.get('position', len(data) + extra_field_index)
                    if not 'value' in value:
                        raise CsvException("If a positional extra argument is defined, a value key should be present.")
                    line.insert(position, value['value'])
                else:
                    raise ImproperlyConfigured("Extra field should be a string or a list")

    def import_data(self,data):
        lines = []
        self.get_class_delimiter()
        line_number = 0
        for line in csv.reader(data,delimiter = self.delimiter):
            self.process_line(data, line, lines, line_number)
            line_number += 1
        return lines
        
        
    def process_line(self, data, line, lines, line_number):
        self.process_extra_fields(data, line)
        try :
            self.layout.process_line(lines, line, self.csvModel, delimiter = self.delimiter)
        except SkipRow:
            pass
        except ForeignKeyFieldError, e:
            raise CsvFieldDataException(line_number, field_error =  e.message, model = e.model, value = e.value)
        except ValueError, e:
            if line_number == 0 and self.csvModel.has_header():
                pass
            else:
                raise CsvDataException(line_number, field_error =  e.message)
        except IndexError,e :
            raise CsvDataException(line_number, error = "Number of fields invalid")
        
        
        
    def get_class_delimiter(self):
        if not self.delimiter and hasattr(self.csvModel, 'Meta') and hasattr(self.csvModel.Meta, 'delimiter'):
            self.delimiter = self.csvModel.Meta.delimiter
            
    def import_from_filename(self,filename):
        csv_file = open(filename)
        return self.import_from_file(csv_file)
    
    def import_from_file(self,csv_file):
        self.get_class_delimiter()
        if not self.delimiter:
            dialect = csv.Sniffer().sniff(csv_file.read(1024))
            self.delimiter = dialect.delimiter
        csv_file.seek(0)
        return self.import_data(csv_file)


    def __getitem__(self, item):
        return self.lines[item]

    def __iter__(self):
        return self.lines.__iter__()





#!/usr/bin/env python
import os
from distutils.core import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='django-csv-importer',
      version='0.1.3.4',
      description='Convert csv files into python object or django model',
      author='Anthony Tresontani',
      author_email='dev.tresontani@gmail.com',
      long_description =read('README.txt'),
      license = "BSD",
      keywords = "CSV Django loader",
      packages=['csvImporter'],
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
     )
